@@ -73,6 +73,8 @@ enum
   V4L2_STD_OBJECT_PROPS,
 };
 
+static GSList *gst_v4l2_object_get_format_list (GstV4l2Object * v4l2object);
+
 #if 0
 G_LOCK_DEFINE_STATIC (probe_lock);
 
@@ -593,6 +595,10 @@ gst_v4l2_object_destroy (GstV4l2Object * v4l2object)
     gst_v4l2_object_clear_format_list (v4l2object);
   }
 
+  if (v4l2object->probed_caps) {
+    gst_caps_unref (v4l2object->probed_caps);
+  }
+
   g_free (v4l2object);
 }
 
@@ -1263,7 +1269,7 @@ failed:
  * Get the list of supported capture formats, a list of
  * <code>struct v4l2_fmtdesc</code>.
  */
-GSList *
+static GSList *
 gst_v4l2_object_get_format_list (GstV4l2Object * v4l2object)
 {
   if (!v4l2object->formats)
@@ -1913,7 +1919,7 @@ sort_by_frame_size (GstStructure * s1, GstStructure * s2)
 }
 #endif
 
-GstCaps *
+static GstCaps *
 gst_v4l2_object_probe_caps_for_format (GstV4l2Object * v4l2object,
     guint32 pixelformat, const GstStructure * template)
 {
@@ -2663,3 +2669,53 @@ invalid_buffer:
     return FALSE;
   }
 }
+
+GstCaps *
+gst_v4l2_object_get_caps (GstV4l2Object * v4l2object, GstCaps * filter)
+{
+  GstCaps *ret;
+  GSList *walk;
+  GSList *formats;
+
+  if (v4l2object->probed_caps == NULL) {
+    formats = gst_v4l2_object_get_format_list (v4l2object);
+
+    ret = gst_caps_new_empty ();
+
+    for (walk = formats; walk; walk = walk->next) {
+      struct v4l2_fmtdesc *format;
+      GstStructure *template;
+
+      format = (struct v4l2_fmtdesc *) walk->data;
+
+      template = gst_v4l2_object_v4l2fourcc_to_structure (format->pixelformat);
+
+      if (template) {
+        GstCaps *tmp;
+
+        tmp = gst_v4l2_object_probe_caps_for_format (v4l2object,
+            format->pixelformat, template);
+        if (tmp)
+          gst_caps_append (ret, tmp);
+
+        gst_structure_free (template);
+      } else {
+        GST_DEBUG_OBJECT (v4l2object->element, "unknown format %u",
+            format->pixelformat);
+      }
+    }
+    v4l2object->probed_caps = ret;
+  }
+
+  if (filter) {
+    ret = gst_caps_intersect_full (filter, v4l2object->probed_caps,
+        GST_CAPS_INTERSECT_FIRST);
+  } else {
+    ret = gst_caps_ref (v4l2object->probed_caps);
+  }
+
+  GST_INFO_OBJECT (v4l2object->element, "probed caps: %p", ret);
+  LOG_CAPS (v4l2object->element, ret);
+
+  return ret;
+}
